@@ -1,14 +1,13 @@
-from enum import Enum
 import numpy as np
 import napari
 from typing import Callable
 from functools import wraps
 from toolz import curry
 import inspect
-import numpy as np
-import napari
 from napari_tools_menu import register_function
 from napari_time_slicer import time_slicer
+from napari_skimage_regionprops._all_frames import analyze_all_frames
+from napari_skimage_regionprops import relabel as _relabel
 
 @curry
 def plugin_function(
@@ -62,11 +61,17 @@ def plugin_function(
     return worker_function
 
 
-
 @register_function(menu="Filtering / noise removal > Median (n-SimpleITK)")
 @time_slicer
 @plugin_function
-def median_filter(image:napari.types.ImageData, radius_x: int = 1, radius_y: int = 1, radius_z: int = 0, viewer: napari.Viewer = None) -> napari.types.ImageData:
+def median_filter(image:napari.types.ImageData, radius_x: int = 1, radius_y: int = 1, radius_z: int = 0) -> napari.types.ImageData:
+    """
+    The median-filter allows removing noise from images. While locally averaging intensity, it
+    is an edge-preserving filter.
+
+    It is equal to a percentile-filter with percentile==50.
+    In case applying the filter takes to much time, consider using a Gaussian blur instead.
+    """
     import SimpleITK as sitk
     return sitk.Median(image, [radius_x, radius_y, radius_z])
 
@@ -74,7 +79,7 @@ def median_filter(image:napari.types.ImageData, radius_x: int = 1, radius_y: int
 @register_function(menu="Filtering / noise removal > Gaussian (n-SimpleITK)")
 @time_slicer
 @plugin_function
-def gaussian_blur(image:napari.types.ImageData, variance_x: float = 1, variance_y: float = 1, variance_z: float = 0, viewer: napari.Viewer = None) -> napari.types.ImageData:
+def gaussian_blur(image:napari.types.ImageData, variance_x: float = 1, variance_y: float = 1, variance_z: float = 0) -> napari.types.ImageData:
     import SimpleITK as sitk
     return sitk.DiscreteGaussian(image, variance=[variance_x, variance_y, variance_z])
 
@@ -318,7 +323,7 @@ def threshold_isodata(image:napari.types.ImageData, viewer: napari.Viewer = None
 @plugin_function
 def threshold_triangle(image:napari.types.ImageData, viewer: napari.Viewer = None) -> napari.types.LabelsData:
     """
-    Binarize an image according to Otsu's method.
+    Binarize an image according to the Triangle method.
 
     Parameters
     ----------
@@ -457,8 +462,7 @@ def simple_linear_iterative_clustering(image:napari.types.ImageData,
                                        spatial_proximity_weight:float = 10,
                                        grid_size_x:int=50,
                                        grid_size_y:int=50,
-                                       grid_size_z:int=50,
-                                       viewer: napari.Viewer = None) -> napari.types.LabelsData:
+                                       grid_size_z:int=50) -> napari.types.LabelsData:
     import SimpleITK as sitk
     return sitk.SLIC(image,
                      maximumNumberOfIterations=maximum_number_of_iterations,
@@ -469,8 +473,7 @@ def simple_linear_iterative_clustering(image:napari.types.ImageData,
 @register_function(menu="Segmentation / labeling > Scalar image K-means clustering (n-SimpleITK)")
 @time_slicer
 @plugin_function
-def scalar_image_k_means_clustering(image:napari.types.ImageData,
-                                       viewer: napari.Viewer = None) -> napari.types.LabelsData:
+def scalar_image_k_means_clustering(image:napari.types.ImageData) -> napari.types.LabelsData:
     import SimpleITK as sitk
     return sitk.ScalarImageKmeans(image)
 
@@ -478,7 +481,10 @@ def scalar_image_k_means_clustering(image:napari.types.ImageData,
 @register_function(menu="Segmentation / labeling > Connected component labeling (n-SimpleITK)")
 @time_slicer
 @plugin_function
-def connected_component_labeling(binary_image:napari.types.LabelsData, viewer: napari.Viewer = None) -> napari.types.LabelsData:
+def connected_component_labeling(binary_image:napari.types.LabelsData) -> napari.types.LabelsData:
+    """
+    Takes a binary image and produces a label image with all separated objects labeled differently.
+    """
     import SimpleITK as sitk
     return sitk.ConnectedComponent(binary_image)
 
@@ -486,13 +492,20 @@ def connected_component_labeling(binary_image:napari.types.LabelsData, viewer: n
 @register_function(menu="Segmentation / labeling > Touching objects labeling (n-SimpleITK)")
 @time_slicer
 @plugin_function
-def touching_objects_labeling(binary_image:napari.types.LabelsData, viewer: napari.Viewer = None) -> napari.types.LabelsData:
+def touching_objects_labeling(binary_image:napari.types.LabelsData) -> napari.types.LabelsData:
     """
-    Takes a binary image an splits touching objects into multiple similar to the Watershed segmentation in ImageJ.
+    Takes a binary image an splits touching objects into multiple similar to the Watershed segmentation in ImageJ [1].
+
+    This allows cutting connected objects such as not to dense nuclei. If the nuclei are too dense,
+    consider using stardist [2] or cellpose [3].
 
     See also
     --------
-    ..[0] http://insightsoftwareconsortium.github.io/SimpleITK-Notebooks/Python_html/32_Watersheds_Segmentation.html#Multi-label-Morphology
+    .. [0] http://insightsoftwareconsortium.github.io/SimpleITK-Notebooks/Python_html/32_Watersheds_Segmentation.html#Multi-label-Morphology
+    .. [1] https://imagej.nih.gov/ij/docs/menus/process.html#watershed
+    .. [2] https://www.napari-hub.org/plugins/stardist-napari
+    .. [3] https://www.napari-hub.org/plugins/cellpose-napari
+
     """
     import SimpleITK as sitk
     distance_map = sitk.SignedMaurerDistanceMap(binary_image, insideIsPositive=False, squaredDistance=False,
@@ -506,7 +519,7 @@ def touching_objects_labeling(binary_image:napari.types.LabelsData, viewer: napa
 @register_function(menu="Segmentation / labeling > Watershed Otsu labeling (n-SimpleITK)")
 @time_slicer
 @plugin_function
-def watershed_otsu_labeling(image:napari.types.ImageData, spot_sigma: float = 2, outline_sigma: float = 2, watershed_level:float = 10, viewer: napari.Viewer = None) -> napari.types.LabelsData:
+def watershed_otsu_labeling(image:napari.types.ImageData, spot_sigma: float = 2, outline_sigma: float = 2, watershed_level:float = 10) -> napari.types.LabelsData:
     """
     The two sigma parameters and the level allow tuning the segmentation result. The first sigma controls how close detected cells
     can be (spot_sigma) and the second controls how precise segmented objects are outlined (outline_sigma). Under the
@@ -785,25 +798,46 @@ def label_contour(label_image:napari.types.LabelsData, fully_connected: bool = T
 
 
 @register_function(menu="Measurement > Measurements (n-SimpleITK)")
-@time_slicer
 def label_statistics(
         intensity_image: napari.types.ImageData,
         label_image: napari.types.LabelsData,
-        napari_viewer: napari.Viewer = None,
         size: bool = True, intensity: bool = True, perimeter: bool = False,
-        shape: bool = False, position: bool = False, moments: bool = False):
-    """
+        shape: bool = False, position: bool = False, moments: bool = False,
+        napari_viewer: napari.Viewer = None) -> "pandas.DataFrame":
+    """Measure intensity/shape/... statistics per label
+
+    Parameters
+    ----------
+    intensity_image: ndarray, optional
+        Can be None
+    label_image: ndarray
+        Must be subsequently labeled
+    size: bool, optional
+    intensity: bool, optional
+    perimeter: bool, optional
+    shape: bool, optional
+    position: bool, optional
+    moments: bool, optional
+    napari_viewer: napari.Viewer, optional
+
+    Returns
+    -------
+    pandas DataFrame, in case napari_viewr is None, otherwise the DataFrame will be added to
+    the passed label_image's layer.features
+
+
     See Also
     --------
     ..[0] https://simpleitk.org/doxygen/latest/html/classitk_1_1simple_1_1LabelShapeStatisticsImageFilter
     ..[1] http://insightsoftwareconsortium.github.io/SimpleITK-Notebooks/Python_html/35_Segmentation_Shape_Analysis.html
     """
     import SimpleITK as sitk
-    sitk_intensity_image = sitk.GetImageFromArray(intensity_image)
-    sitk_label_image = sitk.GetImageFromArray(label_image)
 
-    intensity_stats = sitk.LabelStatisticsImageFilter()
-    intensity_stats.Execute(sitk_intensity_image, sitk_label_image)
+    sitk_label_image = sitk.GetImageFromArray(label_image.astype(int))
+    if intensity:
+        sitk_intensity_image = sitk.GetImageFromArray(intensity_image)
+        intensity_stats = sitk.LabelStatisticsImageFilter()
+        intensity_stats.Execute(sitk_intensity_image, sitk_label_image)
 
     shape_stats = sitk.LabelShapeStatisticsImageFilter()
     shape_stats.SetComputeFeretDiameter(True)
@@ -835,19 +869,15 @@ def label_statistics(
 
         if shape:
             _append_to_column(results, "elongation", shape_stats.GetElongation(l))
-
             _append_to_column(results, "feret_diameter", shape_stats.GetFeretDiameter(l))
             _append_to_column(results, "flatness", shape_stats.GetFlatness(l))
-
             _append_to_column(results, "roundness", shape_stats.GetRoundness(l))
 
         if size:
             for i, value in enumerate(shape_stats.GetEquivalentEllipsoidDiameter(l)):
                 _append_to_column(results, "equivalent_ellipsoid_diameter_" + str(i), value)
-
             _append_to_column(results, "equivalent_spherical_perimeter", shape_stats.GetEquivalentSphericalPerimeter(l))
             _append_to_column(results, "equivalent_spherical_radius", shape_stats.GetEquivalentSphericalRadius(l))
-
             _append_to_column(results, "number_of_pixels", shape_stats.GetNumberOfPixels(l))
             _append_to_column(results, "number_of_pixels_on_border", shape_stats.GetNumberOfPixelsOnBorder(l))
 
@@ -880,7 +910,11 @@ def label_statistics(
         from napari_skimage_regionprops import add_table
         add_table(labels_layer, napari_viewer)
     else:
-        return results
+        import pandas
+        return pandas.DataFrame(results)
+
+label_statistics_in_all_frames = analyze_all_frames(label_statistics)
+register_function(label_statistics_in_all_frames, menu="Measurement > Measurements of all frames (n-SimpleITK)")
 
 
 def _append_to_column(dictionary, column_name, value):
@@ -893,7 +927,7 @@ def _append_to_column(dictionary, column_name, value):
 @time_slicer
 def pixel_count_map(label_image:napari.types.LabelsData, viewer: napari.Viewer = None) -> napari.types.ImageData:
     statistics = label_statistics(intensity_image=label_image, label_image=label_image,size=True, intensity=False)
-    measurement_vector = statistics["number_of_pixels"]
+    measurement_vector = statistics["number_of_pixels"].tolist()
     return _relabel(label_image, measurement_vector)
 
 
@@ -901,7 +935,7 @@ def pixel_count_map(label_image:napari.types.LabelsData, viewer: napari.Viewer =
 @time_slicer
 def elongation_map(label_image:napari.types.LabelsData, viewer: napari.Viewer = None) -> napari.types.ImageData:
     statistics = label_statistics(intensity_image=label_image, label_image=label_image,size=False, intensity=False, shape=True)
-    measurement_vector = statistics["elongation"]
+    measurement_vector = statistics["elongation"].tolist()
     return _relabel(label_image, measurement_vector)
 
 
@@ -909,7 +943,7 @@ def elongation_map(label_image:napari.types.LabelsData, viewer: napari.Viewer =
 @time_slicer
 def feret_diameter_map(label_image:napari.types.LabelsData, viewer: napari.Viewer = None) -> napari.types.ImageData:
     statistics = label_statistics(intensity_image=label_image, label_image=label_image,size=False, intensity=False, shape=True)
-    measurement_vector = statistics["feret_diameter"]
+    measurement_vector = statistics["feret_diameter"].tolist()
     return _relabel(label_image, measurement_vector)
 
 
@@ -917,17 +951,5 @@ def feret_diameter_map(label_image:napari.types.LabelsData, viewer: napari.Viewe
 @time_slicer
 def roundness_map(label_image:napari.types.LabelsData, viewer: napari.Viewer = None) -> napari.types.ImageData:
     statistics = label_statistics(intensity_image=label_image, label_image=label_image,size=False, intensity=False, shape=True)
-    measurement_vector = statistics["roundness"]
+    measurement_vector = statistics["roundness"].tolist()
     return _relabel(label_image, measurement_vector)
-
-
-def _relabel(labels, measurements):
-    try:
-        import pyclesperanto_prototype as cle;
-        return cle.pull(cle.replace_intensities(labels, numpy.asarray([0] + measurements)))
-    except ImportError:
-        return _relabel_numpy(labels, measurements)
-
-
-def _relabel_numpy(image, measurements):
-    return np.take(np.array([0] + measurements), image)
