@@ -133,7 +133,7 @@ def test_statistics():
     ])
     labels = image
 
-    result = label_statistics(image, labels, None, size=True, intensity=True, perimeter=True, shape=True, position=True, moments=True )
+    result = label_statistics(image, labels, size=True, intensity=True, perimeter=True, shape=True, position=True, moments=True )
 
     print(result)
 
@@ -143,6 +143,7 @@ def test_statistics():
     for k, v in reference.items():
         assert np.allclose(result[k], reference[k], 0.001)
 
+
 def test_statistics_with_viewer(make_napari_viewer):
     from napari_simpleitk_image_processing import label_statistics
 
@@ -158,19 +159,74 @@ def test_statistics_with_viewer(make_napari_viewer):
 
     labels_layer = viewer.add_labels(labels, name="test1")
 
-    label_statistics(image, labels, viewer, size=True, intensity=True, perimeter=True, shape=True, position=True, moments=True )
+    label_statistics(image, labels, napari_viewer=viewer, size=True, intensity=True, perimeter=True, shape=True, position=True, moments=True )
 
     result = labels_layer.properties
     print(result)
 
     for k, v in result.items():
-        assert np.allclose(result[k], reference[k], 0.001)
+        if k != "index":
+            assert np.allclose(result[k], reference[k], 0.001)
 
     for k, v in reference.items():
-        assert np.allclose(result[k], reference[k], 0.001)
+        if k != "index":
+            assert np.allclose(result[k], reference[k], 0.001)
 
 
+def test_statistics_with_viewer_and_dask(make_napari_viewer):
+    from napari_simpleitk_image_processing import label_statistics
+    import dask.array as da
+
+    viewer = make_napari_viewer()
+
+    image = da.asarray(np.asarray([
+        [0, 1, 2, 3],
+        [0, 1, 2, 3],
+        [4, 4, 4, 4],
+        [5, 5, 0, 0]
+    ]))
+    labels = da.asarray(image.astype(int))
+
+    labels_layer = viewer.add_labels(labels, name="test1")
+
+    label_statistics(image, labels, napari_viewer=viewer, size=True, intensity=True, perimeter=True, shape=True, position=True, moments=True )
+
+    result = labels_layer.properties
+    print(result)
+
+    for k, v in result.items():
+        if k != "index":
+            assert np.allclose(result[k], reference[k], 0.001)
+
+    for k, v in reference.items():
+        if k != "index":
+            assert np.allclose(result[k], reference[k], 0.001)
+
 
 def test_napari_api():
     from napari_simpleitk_image_processing import napari_experimental_provide_function
     napari_experimental_provide_function()
+
+def test_label_statistics():
+    from napari_simpleitk_image_processing import label_statistics
+
+    image = np.asarray(np.asarray([
+        [0, 1, 2, 3],
+        [0, 1, 2, 3],
+        [4, 4, 4, 4],
+        [5, 5, 0, 0]
+    ]))
+    labels = image.astype(int)
+
+    # test all
+    res = label_statistics(image, labels, size=True, intensity=True, perimeter=True, shape=True, position=True, moments=True )
+    assert "perimeter" in res.keys()
+    assert "minimum" in res.keys()
+    # test anything but intensity
+    res = label_statistics(None, labels, size=True, intensity=False, perimeter=True, shape=True, position=True, moments=True)
+    assert "perimeter" in res.keys()
+    assert "minimum" not in res.keys()
+    # test intensity only
+    res = label_statistics(image, labels, size=False, intensity=True, perimeter=False, shape=False, position=False, moments=False)
+    assert "perimeter" not in res.keys()
+    assert "minimum" in res.keys()
